==> cdc-status-1.2.7-sources.jar: com/xforceplus/ultraman/extensions/cdc/status/query/WaitForStrategy.java <==
package com.xforceplus.ultraman.extensions.cdc.status.query;

import com.xforceplus.ultraman.extensions.cdc.status.StatusService;
import com.xforceplus.ultraman.metadata.entity.IEntityClass;
import com.xforceplus.ultraman.sdk.core.calcite.oqs.DataQueryProvider;
import com.xforceplus.ultraman.sdk.core.calcite.oqs.strategy.QueryStrategy;
import io.vavr.Tuple;
import io.vavr.Tuple2;
import org.apache.calcite.DataContext;
import org.apache.calcite.rel.RelFieldCollation;
import org.apache.calcite.rel.RelNode;
import org.apache.calcite.rel.core.AggregateCall;
import org.apache.calcite.rel.hint.RelHint;
import org.apache.calcite.rel.type.RelDataType;
import org.apache.calcite.rel.type.StructKind;
import org.apache.calcite.rex.RexNode;
import org.apache.calcite.util.Pair;
import org.springframework.core.Ordered;

import java.time.Instant;
import java.time.LocalDateTime;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Optional;

import static com.xforceplus.ultraman.metadata.values.DateTimeValue.ZONE_ID;

/**
 * TODO
 */
public class WaitForStrategy implements QueryStrategy {

    private StatusService service;

    public WaitForStrategy(StatusService service) {
        this.service = service;
    }

    @Override
    public String name() {
        return "wait_for";
    }

    @Override
    public int order() {
        return Ordered.LOWEST_PRECEDENCE;
    }

    @Override
    public boolean accept(DataQueryProvider.QueryProviderType type) {
        return type == DataQueryProvider.QueryProviderType.INDEX;
    }

    @Override
    public void doWithInput(String app, IEntityClass entityClass, String profile, RelDataType type, List<RexNode> ops, List<Map.Entry<String, Tuple2<StructKind, Class>>> fields
            , List<Pair<RexNode, String>> projects, List<Map.Entry<String, RelFieldCollation.Direction>> sort
            , Long offset, Long fetch, List<String> groupBy, List<AggregateCall> aggs, List<RelHint> hints
            , RelNode rawTree, DataContext dataContext) {

        if(hints != null) {
            Optional<RelHint> waitfor = hints.stream().filter(x -> x.hintName.equalsIgnoreCase("wait_for")).findFirst();
            if(waitfor.isPresent()) {
                //TODO simple
                Instant instant = LocalDateTime.now().atZone(ZONE_ID).toInstant();
                long currentTime = instant.toEpochMilli();
                service.query(Collections.singletonList(Tuple.of(entityClass.id(), profile)), currentTime);
            }
        }
    }
}

==> cdc-status-1.2.40-sources.jar: com/xforceplus/ultraman/extensions/cdc/status/query/WaitForStrategy.java <==
package com.xforceplus.ultraman.extensions.cdc.status.query;

import com.xforceplus.ultraman.extensions.cdc.status.StatusService;
import com.xforceplus.ultraman.metadata.entity.IEntityClass;
import com.xforceplus.ultraman.sdk.core.calcite.oqs.DataQueryProvider;
import com.xforceplus.ultraman.sdk.core.calcite.oqs.strategy.QueryStrategy;
import io.vavr.Tuple;
import io.vavr.Tuple2;
import org.apache.calcite.DataContext;
import org.apache.calcite.rel.RelFieldCollation;
import org.apache.calcite.rel.RelNode;
import org.apache.calcite.rel.core.AggregateCall;
import org.apache.calcite.rel.hint.RelHint;
import org.apache.calcite.rel.type.RelDataType;
import org.apache.calcite.rel.type.StructKind;
import org.apache.calcite.rex.RexNode;
import org.apache.calcite.util.Pair;
import org.springframework.core.Ordered;

import java.time.Instant;
import java.time.LocalDateTime;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Optional;

import static com.xforceplus.ultraman.metadata.values.DateTimeValue.ZONE_ID;

/**
 * TODO
 */
public class WaitForStrategy implements QueryStrategy {

    private StatusService service;

    public WaitForStrategy(StatusService service) {
        this.service = service;
    }

    @Override
    public String name() {
        return "wait_for";
    }

    @Override
    public int order() {
        return Ordered.LOWEST_PRECEDENCE;
    }

    @Override
    public boolean accept(DataQueryProvider.QueryProviderType type) {
        return type == DataQueryProvider.QueryProviderType.INDEX;
    }

    @Override
    public void doWithInput(DataQueryProvider dataQueryProvider, String app, IEntityClass entityClass, String profile, RelDataType type, List<RexNode> ops, List<Map.Entry<String, Tuple2<StructKind, Class>>> fields
            , List<Pair<RexNode, String>> projects, List<Map.Entry<String, RelFieldCollation.Direction>> sort
            , Long offset, Long fetch, List<String> groupBy, List<AggregateCall> aggs, List<RelHint> hints
            , RelNode rawTree, DataContext dataContext) {

        if(hints != null) {
            Optional<RelHint> waitfor = hints.stream().filter(x -> x.hintName.equalsIgnoreCase("wait_for")).findFirst();
            if(waitfor.isPresent() && dataQueryProvider.type() == DataQueryProvider.QueryProviderType.INDEX) {
                //TODO simple
                Instant instant = LocalDateTime.now().atZone(ZONE_ID).toInstant();
                long currentTime = instant.toEpochMilli();
                service.query(Collections.singletonList(Tuple.of(entityClass.id(), profile)), currentTime);
            }
        }
    }
}
